# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

import copy
import pytest
from iced_x86 import *

def test_ctor():
	instr = Instruction()
	assert instr.code == Code.INVALID
	assert instr.code_size == CodeSize.UNKNOWN
	assert instr.len == 0
	assert instr.ip == 0

def test_eq_ne_hash():
	decodera = Decoder(64, b"\xC4\xE3\x49\x48\x10\x41" b"\xC4\xE3\x49\x48\x10\x42", ip=0x1234_5678_9ABC_DEF1)
	decoderb = Decoder(64, b"\xC4\xE3\x49\x48\x10\x41", ip=0xABCD_EF01_1234_5678)
	instr1 = decodera.decode()
	instr2 = decodera.decode()
	instr3 = decoderb.decode()

	assert instr1 == instr1
	assert not (instr1 != instr1)
	assert instr1 != instr2
	assert not (instr1 == instr2)
	assert instr1 == instr3
	assert not (instr1 != instr3)
	assert hash(instr1) == hash(instr3)

	assert instr1 != 1
	assert instr1 != 1.23
	assert instr1 != None
	assert instr1 != []
	assert instr1 != {}
	assert instr1 != (1, 2)

	assert not (instr1 == 1)
	assert not (instr1 == 1.23)
	assert not (instr1 == None)
	assert not (instr1 == [])
	assert not (instr1 == {})
	assert not (instr1 == (1, 2))

def test_invalid():
	instr = Instruction()
	assert not instr
	assert instr.is_invalid
	instr.code = Code.ADD_AL_IMM8
	assert instr
	assert not instr.is_invalid

@pytest.mark.parametrize("copy_instr", [
	lambda instr: copy.copy(instr),
	lambda instr: copy.deepcopy(instr),
	lambda instr: instr.copy(),
])
def test_copy_deepcopy_mcopy(copy_instr):
	decoder = Decoder(64, b"\xC4\xE3\x49\x48\x10\x41", ip=0x1234_5678_9ABC_DEF1)
	instr = decoder.decode()
	instr2 = copy_instr(instr)
	assert instr is not instr2
	assert id(instr) != id(instr2)

	assert instr == instr2
	assert not (instr != instr2)
	assert instr.eq_all_bits(instr2)
	assert instr2.eq_all_bits(instr)
	assert hash(instr) == hash(instr2)

	instr2.ip += 1
	assert instr == instr2
	assert not (instr != instr2)
	assert not instr.eq_all_bits(instr2)
	assert not instr2.eq_all_bits(instr)
	assert hash(instr) == hash(instr2)

def test_some_props1():
	decoder = Decoder(64, b"\xC4\xE3\x49\x48\x10\x41", ip=0x1234_5678_9ABC_DEF1)
	instr = decoder.decode()

	assert instr.len == 6
	assert len(instr) == 6

	assert instr.ip16 == 0xDEF1
	assert instr.ip32 == 0x9ABC_DEF1
	assert instr.ip == 0x1234_5678_9ABC_DEF1
	assert instr.next_ip16 == 0xDEF7
	assert instr.next_ip32 == 0x9ABC_DEF7
	assert instr.next_ip == 0x1234_5678_9ABC_DEF7

	instr.ip = 0x9ABC_DEF0_1234_5678
	assert instr.ip16 == 0x5678
	assert instr.ip32 == 0x1234_5678
	assert instr.ip == 0x9ABC_DEF0_1234_5678
	assert instr.next_ip16 == 0x567E
	assert instr.next_ip32 == 0x1234_567E
	assert instr.next_ip == 0x9ABC_DEF0_1234_567E

	instr.ip = 0x9ABC_DEF0_1234_5678
	instr.ip32 = 0xABCD_EF01
	assert instr.ip16 == 0xEF01
	assert instr.ip32 == 0xABCD_EF01
	assert instr.ip == 0xABCD_EF01
	assert instr.next_ip16 == 0xEF07
	assert instr.next_ip32 == 0xABCD_EF07
	assert instr.next_ip == 0xABCD_EF07

	instr.ip = 0x9ABC_DEF0_1234_5678
	instr.ip16 = 0xABCD
	assert instr.ip16 == 0xABCD
	assert instr.ip32 == 0xABCD
	assert instr.ip == 0xABCD
	assert instr.next_ip16 == 0xABD3
	assert instr.next_ip32 == 0xABD3
	assert instr.next_ip == 0xABD3

	instr.next_ip = 0x9ABC_DEF0_1234_5678
	assert instr.ip16 == 0x5672
	assert instr.ip32 == 0x1234_5672
	assert instr.ip == 0x9ABC_DEF0_1234_5672
	assert instr.next_ip16 == 0x5678
	assert instr.next_ip32 == 0x1234_5678
	assert instr.next_ip == 0x9ABC_DEF0_1234_5678

	instr.next_ip = 0x9ABC_DEF0_1234_5678
	instr.next_ip32 = 0xABCD_EF01
	assert instr.ip16 == 0xEEFB
	assert instr.ip32 == 0xABCD_EEFB
	assert instr.ip == 0xABCD_EEFB
	assert instr.next_ip16 == 0xEF01
	assert instr.next_ip32 == 0xABCD_EF01
	assert instr.next_ip == 0xABCD_EF01

	instr.next_ip = 0x9ABC_DEF0_1234_5678
	instr.next_ip16 = 0xABCD
	assert instr.ip16 == 0xABC7
	assert instr.ip32 == 0xABC7
	assert instr.ip == 0xABC7
	assert instr.next_ip16 == 0xABCD
	assert instr.next_ip32 == 0xABCD
	assert instr.next_ip == 0xABCD

	assert instr.code == Code.VEX_VPERMIL2PS_XMM_XMM_XMMM128_XMM_IMM4
	assert instr.mnemonic == Mnemonic.VPERMIL2PS
	assert instr.op_count == 5
	instr.code = Code.ADD_AL_IMM8
	assert instr.code == Code.ADD_AL_IMM8
	assert instr.mnemonic == Mnemonic.ADD
	assert instr.op_count == 2

	assert instr.len == 6
	assert len(instr) == 6
	instr.len = 1
	assert instr.len == 1
	assert len(instr) == 1

def test_some_props2():
	decoder = Decoder(64, b"\x00\xCE", ip=0x1234_5678_9ABC_DEF1)
	instr = decoder.decode()

	assert not instr.has_xacquire_prefix
	instr.has_xacquire_prefix = True
	assert instr.has_xacquire_prefix

	assert not instr.has_xrelease_prefix
	instr.has_xrelease_prefix = True
	assert instr.has_xrelease_prefix

	assert not instr.has_rep_prefix
	assert not instr.has_repe_prefix
	instr.has_rep_prefix = True
	assert instr.has_rep_prefix
	assert instr.has_repe_prefix
	instr.has_repe_prefix = False
	assert not instr.has_rep_prefix
	assert not instr.has_repe_prefix

	assert not instr.has_repne_prefix
	instr.has_repne_prefix = True
	assert instr.has_repne_prefix

	assert not instr.has_lock_prefix
	instr.has_lock_prefix = True
	assert instr.has_lock_prefix

	assert instr.op_mask == Register.NONE
	assert not instr.has_op_mask
	instr.op_mask = Register.K2
	assert instr.op_mask == Register.K2
	assert instr.has_op_mask

	assert not instr.zeroing_masking
	assert instr.merging_masking
	instr.zeroing_masking = True
	assert instr.zeroing_masking
	assert not instr.merging_masking
	instr.merging_masking = True
	assert not instr.zeroing_masking
	assert instr.merging_masking

	assert instr.rounding_control == RoundingControl.NONE
	instr.rounding_control = RoundingControl.ROUND_TO_NEAREST
	assert instr.rounding_control == RoundingControl.ROUND_TO_NEAREST

	assert not instr.suppress_all_exceptions
	instr.suppress_all_exceptions = True
	assert instr.suppress_all_exceptions

	assert not instr.is_privileged
	assert not instr.is_save_restore_instruction

@pytest.mark.parametrize("bitness, code_size, data", [
	(16, CodeSize.CODE16, b"\x90"),
	(32, CodeSize.CODE32, b"\x90"),
	(64, CodeSize.CODE64, b"\x90"),
])
def test_code_size(bitness, code_size, data):
	instr = Decoder(bitness, data).decode()
	assert instr.code_size == code_size
	for new_size in [CodeSize.UNKNOWN, CodeSize.CODE16, CodeSize.CODE32, CodeSize.CODE64]:
		instr.code_size = new_size
		assert instr.code_size == new_size

def test_op_kind():
	decoder = Decoder(64, b"\xC4\xE3\x49\x48\x10\x41", ip=0x1234_5678_9ABC_DEF1)
	instr = decoder.decode()

	assert instr.op0_kind == OpKind.REGISTER
	assert instr.op1_kind == OpKind.REGISTER
	assert instr.op2_kind == OpKind.MEMORY
	assert instr.op3_kind == OpKind.REGISTER
	assert instr.op4_kind == OpKind.IMMEDIATE8

	assert instr.op0_kind == instr.op_kind(0)
	assert instr.op1_kind == instr.op_kind(1)
	assert instr.op2_kind == instr.op_kind(2)
	assert instr.op3_kind == instr.op_kind(3)
	assert instr.op4_kind == instr.op_kind(4)

	instr.op0_kind = OpKind.FAR_BRANCH16
	instr.op1_kind = OpKind.FAR_BRANCH32
	instr.op2_kind = OpKind.IMMEDIATE16
	instr.op3_kind = OpKind.IMMEDIATE32

	assert instr.op0_kind == OpKind.FAR_BRANCH16
	assert instr.op1_kind == OpKind.FAR_BRANCH32
	assert instr.op2_kind == OpKind.IMMEDIATE16
	assert instr.op3_kind == OpKind.IMMEDIATE32
	with pytest.raises(ValueError):
		instr.op4_kind = OpKind.IMMEDIATE64
	instr.op4_kind = OpKind.IMMEDIATE8

	assert instr.op0_kind == instr.op_kind(0)
	assert instr.op1_kind == instr.op_kind(1)
	assert instr.op2_kind == instr.op_kind(2)
	assert instr.op3_kind == instr.op_kind(3)
	assert instr.op4_kind == instr.op_kind(4)

	instr.set_op_kind(0, OpKind.IMMEDIATE8)
	instr.set_op_kind(1, OpKind.IMMEDIATE8TO16)
	instr.set_op_kind(2, OpKind.IMMEDIATE8TO32)
	instr.set_op_kind(3, OpKind.IMMEDIATE8TO64)

	assert instr.op0_kind == OpKind.IMMEDIATE8
	assert instr.op1_kind == OpKind.IMMEDIATE8TO16
	assert instr.op2_kind == OpKind.IMMEDIATE8TO32
	assert instr.op3_kind == OpKind.IMMEDIATE8TO64
	with pytest.raises(ValueError):
		instr.set_op_kind(4, OpKind.IMMEDIATE64)
	instr.set_op_kind(4, OpKind.IMMEDIATE8)

	assert instr.op0_kind == instr.op_kind(0)
	assert instr.op1_kind == instr.op_kind(1)
	assert instr.op2_kind == instr.op_kind(2)
	assert instr.op3_kind == instr.op_kind(3)
	assert instr.op4_kind == instr.op_kind(4)

def test_op_register():
	decoder = Decoder(64, b"\xC4\xE3\x49\x48\xD3\x40", ip=0x1234_5678_9ABC_DEF1)
	instr = decoder.decode()
	assert instr.op0_kind == OpKind.REGISTER
	assert instr.op1_kind == OpKind.REGISTER
	assert instr.op2_kind == OpKind.REGISTER
	assert instr.op3_kind == OpKind.REGISTER
	assert instr.op4_kind == OpKind.IMMEDIATE8

	assert instr.op0_register == Register.XMM2
	assert instr.op1_register == Register.XMM6
	assert instr.op2_register == Register.XMM3
	assert instr.op3_register == Register.XMM4
	assert instr.op4_register == Register.NONE

	assert instr.op_register(0) == Register.XMM2
	assert instr.op_register(1) == Register.XMM6
	assert instr.op_register(2) == Register.XMM3
	assert instr.op_register(3) == Register.XMM4
	assert instr.op_register(4) == Register.NONE

	instr.op0_register = Register.XMM1
	instr.op1_register = Register.XMM5
	instr.op2_register = Register.XMM7
	instr.op3_register = Register.XMM13
	with pytest.raises(ValueError):
		instr.op4_register = Register.XMM15
	instr.op4_register = Register.NONE

	assert instr.op0_register == Register.XMM1
	assert instr.op1_register == Register.XMM5
	assert instr.op2_register == Register.XMM7
	assert instr.op3_register == Register.XMM13
	assert instr.op4_register == Register.NONE

	assert instr.op_register(0) == Register.XMM1
	assert instr.op_register(1) == Register.XMM5
	assert instr.op_register(2) == Register.XMM7
	assert instr.op_register(3) == Register.XMM13
	assert instr.op_register(4) == Register.NONE

	instr.set_op_register(0, Register.XMM0)
	instr.set_op_register(1, Register.XMM8)
	instr.set_op_register(2, Register.XMM10)
	instr.set_op_register(3, Register.XMM11)
	with pytest.raises(ValueError):
		instr.set_op_register(4, Register.XMM14)
	instr.set_op_register(4, Register.NONE)

	assert instr.op0_register == Register.XMM0
	assert instr.op1_register == Register.XMM8
	assert instr.op2_register == Register.XMM10
	assert instr.op3_register == Register.XMM11
	assert instr.op4_register == Register.NONE

	assert instr.op_register(0) == Register.XMM0
	assert instr.op_register(1) == Register.XMM8
	assert instr.op_register(2) == Register.XMM10
	assert instr.op_register(3) == Register.XMM11
	assert instr.op_register(4) == Register.NONE

def test_mem():
	decoder = Decoder(64, b"\xC4\xE3\x49\x48\x10\x41", ip=0x1234_5678_9ABC_DEF1)
	instr = decoder.decode()

	assert not instr.has_segment_prefix
	assert instr.segment_prefix == Register.NONE
	assert instr.memory_segment == Register.DS
	instr.segment_prefix = Register.GS
	assert instr.has_segment_prefix
	assert instr.segment_prefix == Register.GS
	assert instr.memory_segment == Register.GS

	assert instr.memory_displ_size == 0
	instr.memory_displ_size = 1
	assert instr.memory_displ_size == 1
	instr.memory_displ_size = 2
	assert instr.memory_displ_size == 2
	instr.memory_displ_size = 4
	assert instr.memory_displ_size == 4
	instr.memory_displ_size = 8
	assert instr.memory_displ_size == 8
	instr.memory_displ_size = 0
	assert instr.memory_displ_size == 0

	assert instr.memory_size == MemorySize.PACKED128_FLOAT32

	assert not instr.is_broadcast
	instr.is_broadcast = True
	assert instr.is_broadcast

	assert instr.memory_index_scale == 1
	instr.memory_index_scale = 2
	assert instr.memory_index_scale == 2
	instr.memory_index_scale = 4
	assert instr.memory_index_scale == 4
	instr.memory_index_scale = 8
	assert instr.memory_index_scale == 8
	instr.memory_index_scale = 1
	assert instr.memory_index_scale == 1

	assert instr.memory_displacement == 0
	instr.memory_displacement = 0xFEDC_BA98_7654_3210
	assert instr.memory_displacement == 0xFEDC_BA98_7654_3210
	instr.memory_displacement = 0x1234_5678_9ABC_DEF1
	assert instr.memory_displacement == 0x1234_5678_9ABC_DEF1

	assert instr.memory_base == Register.RAX
	instr.memory_base = Register.R15D
	assert instr.memory_base == Register.R15D

	assert instr.memory_index == Register.NONE
	instr.memory_index = Register.XMM13
	assert instr.memory_index == Register.XMM13

def test_imm8():
	instr = Instruction()
	instr.op0_kind = OpKind.IMMEDIATE8
	instr.immediate8 = 0xFE
	assert instr.immediate8 == 0xFE
	assert instr.immediate(0) == 0xFE
	instr.set_immediate_i32(0, -0x12)
	assert instr.immediate8 == 0xEE
	assert instr.immediate(0) == 0xEE
	instr.set_immediate_u32(0, 0xFE)
	assert instr.immediate8 == 0xFE
	assert instr.immediate(0) == 0xFE
	instr.set_immediate_i64(0, -0x12)
	assert instr.immediate8 == 0xEE
	assert instr.immediate(0) == 0xEE
	instr.set_immediate_u64(0, 0xFE)
	assert instr.immediate8 == 0xFE
	assert instr.immediate(0) == 0xFE

def test_imm8_2nd():
	instr = Instruction()
	instr.op0_kind = OpKind.IMMEDIATE8_2ND
	instr.immediate8_2nd = 0xFE
	assert instr.immediate8_2nd == 0xFE
	assert instr.immediate(0) == 0xFE
	instr.set_immediate_i32(0, -0x12)
	assert instr.immediate8_2nd == 0xEE
	assert instr.immediate(0) == 0xEE
	instr.set_immediate_u32(0, 0xFE)
	assert instr.immediate8_2nd == 0xFE
	assert instr.immediate(0) == 0xFE
	instr.set_immediate_i64(0, -0x12)
	assert instr.immediate8_2nd == 0xEE
	assert instr.immediate(0) == 0xEE
	instr.set_immediate_u64(0, 0xFE)
	assert instr.immediate8_2nd == 0xFE
	assert instr.immediate(0) == 0xFE

def test_imm16():
	instr = Instruction()
	instr.op0_kind = OpKind.IMMEDIATE16
	instr.immediate16 = 0xFEDC
	assert instr.immediate16 == 0xFEDC
	assert instr.immediate(0) == 0xFEDC
	instr.set_immediate_i32(0, -0x1234)
	assert instr.immediate16 == 0xEDCC
	assert instr.immediate(0) == 0xEDCC
	instr.set_immediate_u32(0, 0xFEDC)
	assert instr.immediate16 == 0xFEDC
	assert instr.immediate(0) == 0xFEDC
	instr.set_immediate_i64(0, -0x1234)
	assert instr.immediate16 == 0xEDCC
	assert instr.immediate(0) == 0xEDCC
	instr.set_immediate_u64(0, 0xFEDC)
	assert instr.immediate16 == 0xFEDC
	assert instr.immediate(0) == 0xFEDC

def test_imm32():
	instr = Instruction()
	instr.op0_kind = OpKind.IMMEDIATE32
	instr.immediate32 = 0xFEDC_BA98
	assert instr.immediate32 == 0xFEDC_BA98
	assert instr.immediate(0) == 0xFEDC_BA98
	instr.set_immediate_i32(0, -0x1234_5678)
	assert instr.immediate32 == 0xEDCB_A988
	assert instr.immediate(0) == 0xEDCB_A988
	instr.set_immediate_u32(0, 0xFEDC_BA98)
	assert instr.immediate32 == 0xFEDC_BA98
	assert instr.immediate(0) == 0xFEDC_BA98
	instr.set_immediate_i64(0, -0x1234_5678)
	assert instr.immediate32 == 0xEDCB_A988
	assert instr.immediate(0) == 0xEDCB_A988
	instr.set_immediate_u64(0, 0xFEDC_BA98)
	assert instr.immediate32 == 0xFEDC_BA98
	assert instr.immediate(0) == 0xFEDC_BA98

def test_imm64():
	instr = Instruction()
	instr.op0_kind = OpKind.IMMEDIATE64
	instr.immediate64 = 0xFEDC_BA98_7654_3219
	assert instr.immediate64 == 0xFEDC_BA98_7654_3219
	assert instr.immediate(0) == 0xFEDC_BA98_7654_3219
	instr.set_immediate_i32(0, -0x1234_5678)
	assert instr.immediate64 == 0xFFFF_FFFF_EDCB_A988
	assert instr.immediate(0) == 0xFFFF_FFFF_EDCB_A988
	instr.set_immediate_u32(0, 0xFEDC_BA98)
	assert instr.immediate64 == 0xFEDC_BA98
	assert instr.immediate(0) == 0xFEDC_BA98
	instr.set_immediate_i64(0, -0x1234_5678_9ABC_DEF1)
	assert instr.immediate64 == 0xEDCB_A987_6543_210F
	assert instr.immediate(0) == 0xEDCB_A987_6543_210F
	instr.set_immediate_u64(0, 0xFEDC_BA98_7654_3219)
	assert instr.immediate64 == 0xFEDC_BA98_7654_3219
	assert instr.immediate(0) == 0xFEDC_BA98_7654_3219

def test_imm8to16():
	instr = Instruction()
	instr.op0_kind = OpKind.IMMEDIATE8TO16
	instr.immediate8to16 = -0x12
	assert instr.immediate8to16 == -0x12
	assert instr.immediate(0) == 0xFFFF_FFFF_FFFF_FFEE
	instr.immediate8to16 = 0x12
	assert instr.immediate8to16 == 0x12
	assert instr.immediate(0) == 0x12
	instr.set_immediate_i32(0, -0x12)
	assert instr.immediate8to16 == -0x12
	assert instr.immediate(0) == 0xFFFF_FFFF_FFFF_FFEE
	instr.set_immediate_u32(0, 0x12)
	assert instr.immediate8to16 == 0x12
	assert instr.immediate(0) == 0x12
	instr.set_immediate_i64(0, -0x12)
	assert instr.immediate8to16 == -0x12
	assert instr.immediate(0) == 0xFFFF_FFFF_FFFF_FFEE
	instr.set_immediate_u64(0, 0x12)
	assert instr.immediate8to16 == 0x12
	assert instr.immediate(0) == 0x12

def test_imm8to32():
	instr = Instruction()
	instr.op0_kind = OpKind.IMMEDIATE8TO32
	instr.immediate8to32 = -0x12
	assert instr.immediate8to32 == -0x12
	assert instr.immediate(0) == 0xFFFF_FFFF_FFFF_FFEE
	instr.immediate8to32 = 0x12
	assert instr.immediate8to32 == 0x12
	assert instr.immediate(0) == 0x12
	instr.set_immediate_i32(0, -0x12)
	assert instr.immediate8to32 == -0x12
	assert instr.immediate(0) == 0xFFFF_FFFF_FFFF_FFEE
	instr.set_immediate_u32(0, 0x12)
	assert instr.immediate8to32 == 0x12
	assert instr.immediate(0) == 0x12
	instr.set_immediate_i64(0, -0x12)
	assert instr.immediate8to32 == -0x12
	assert instr.immediate(0) == 0xFFFF_FFFF_FFFF_FFEE
	instr.set_immediate_u64(0, 0x12)
	assert instr.immediate8to32 == 0x12
	assert instr.immediate(0) == 0x12

def test_imm8to64():
	instr = Instruction()
	instr.op0_kind = OpKind.IMMEDIATE8TO64
	instr.immediate8to64 = -0x12
	assert instr.immediate8to64 == -0x12
	assert instr.immediate(0) == 0xFFFF_FFFF_FFFF_FFEE
	instr.immediate8to64 = 0x12
	assert instr.immediate8to64 == 0x12
	assert instr.immediate(0) == 0x12
	instr.set_immediate_i32(0, -0x12)
	assert instr.immediate8to64 == -0x12
	assert instr.immediate(0) == 0xFFFF_FFFF_FFFF_FFEE
	instr.set_immediate_u32(0, 0x12)
	assert instr.immediate8to64 == 0x12
	assert instr.immediate(0) == 0x12
	instr.set_immediate_i64(0, -0x12)
	assert instr.immediate8to64 == -0x12
	assert instr.immediate(0) == 0xFFFF_FFFF_FFFF_FFEE
	instr.set_immediate_u64(0, 0x12)
	assert instr.immediate8to64 == 0x12
	assert instr.immediate(0) == 0x12

def test_imm32to64():
	instr = Instruction()
	instr.op0_kind = OpKind.IMMEDIATE32TO64
	instr.immediate32to64 = -0x1234_5678
	assert instr.immediate32to64 == -0x1234_5678
	assert instr.immediate(0) == 0xFFFF_FFFF_EDCB_A988
	instr.immediate32to64 = 0x1234_5678
	assert instr.immediate32to64 == 0x1234_5678
	assert instr.immediate(0) == 0x1234_5678
	instr.set_immediate_i32(0, -0x1234_5678)
	assert instr.immediate32to64 == -0x1234_5678
	assert instr.immediate(0) == 0xFFFF_FFFF_EDCB_A988
	instr.set_immediate_u32(0, 0x1234_5678)
	assert instr.immediate32to64 == 0x1234_5678
	assert instr.immediate(0) == 0x1234_5678
	instr.set_immediate_i64(0, -0x1234_5678)
	assert instr.immediate32to64 == -0x1234_5678
	assert instr.immediate(0) == 0xFFFF_FFFF_EDCB_A988
	instr.set_immediate_u64(0, 0x1234_5678)
	assert instr.immediate32to64 == 0x1234_5678
	assert instr.immediate(0) == 0x1234_5678

def test_near_br16():
	instr = Instruction()
	instr.op0_kind = OpKind.NEAR_BRANCH16
	instr.near_branch16 = 0xFEDC
	assert instr.near_branch16 == 0xFEDC
	assert instr.near_branch_target == 0xFEDC
	instr.near_branch16 = 0x1234
	assert instr.near_branch16 == 0x1234
	assert instr.near_branch_target == 0x1234

def test_near_br32():
	instr = Instruction()
	instr.op0_kind = OpKind.NEAR_BRANCH32
	instr.near_branch32 = 0xFEDC_BA98
	assert instr.near_branch32 == 0xFEDC_BA98
	assert instr.near_branch_target == 0xFEDC_BA98
	instr.near_branch32 = 0x1234_5678
	assert instr.near_branch32 == 0x1234_5678
	assert instr.near_branch_target == 0x1234_5678

def test_near_br64():
	instr = Instruction()
	instr.op0_kind = OpKind.NEAR_BRANCH64
	instr.near_branch64 = 0xFEDC_BA98_7654_321F
	assert instr.near_branch64 == 0xFEDC_BA98_7654_321F
	assert instr.near_branch_target == 0xFEDC_BA98_7654_321F
	instr.near_branch64 = 0x1234_5678_9ABC_DEF1
	assert instr.near_branch64 == 0x1234_5678_9ABC_DEF1
	assert instr.near_branch_target == 0x1234_5678_9ABC_DEF1

def test_far_br16():
	instr = Instruction()
	instr.op0_kind = OpKind.FAR_BRANCH16
	instr.far_branch16 = 0x1234
	instr.far_branch_selector = 0xABCD
	assert instr.far_branch16 == 0x1234
	assert instr.far_branch_selector == 0xABCD
	instr.far_branch16 = 0xABCD
	instr.far_branch_selector = 0x1234
	assert instr.far_branch16 == 0xABCD
	assert instr.far_branch_selector == 0x1234

def test_far_br32():
	instr = Instruction()
	instr.op0_kind = OpKind.FAR_BRANCH32
	instr.far_branch32 = 0x1234_5678
	instr.far_branch_selector = 0xABCD
	assert instr.far_branch32 == 0x1234_5678
	assert instr.far_branch_selector == 0xABCD
	instr.far_branch32 = 0xABCD_EF01
	instr.far_branch_selector = 0x1234
	assert instr.far_branch32 == 0xABCD_EF01
	assert instr.far_branch_selector == 0x1234

@pytest.mark.parametrize("data", [
	[0x12],
	[0x12, 0x34],
	[0x12, 0x34, 0x56, 0x78, 0x9A, 0xBC, 0xDE, 0xF0, 0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88],
])
def test_db_u(data):
	assert 1 <= len(data) <= 16
	instr = Instruction()
	instr.code = Code.DECLAREBYTE
	instr.declare_data_len = len(data)
	assert instr.declare_data_len == len(data)
	for i, d in enumerate(data):
		instr.set_declare_byte_value(i, d)
	for i, d in enumerate(data):
		assert instr.get_declare_byte_value(i) == d

@pytest.mark.parametrize("data", [
	[0x12],
	[0x12, 0x34],
	[0x12, 0x34, 0x56, 0x78, -0x12, -0x34, -0x56, -0x78, 0x11, 0x22, 0x33, 0x44, -0x11, -0x22, -0x33, -0x44],
])
def test_db_i(data):
	assert 1 <= len(data) <= 16
	instr = Instruction()
	instr.code = Code.DECLAREBYTE
	instr.declare_data_len = len(data)
	assert instr.declare_data_len == len(data)
	for i, d in enumerate(data):
		instr.set_declare_byte_value_i8(i, d)
	for i, d in enumerate(data):
		assert instr.get_declare_byte_value_i8(i) == d

@pytest.mark.parametrize("data", [
	[0x1234],
	[0x1234, 0x89AB],
	[0x1234, 0x89AB, 0x4567, 0xCDEF, 0x1122, 0x8899, 0x3344, 0xAABB],
])
def test_dw_u(data):
	assert 1 <= len(data) <= 8
	instr = Instruction()
	instr.code = Code.DECLAREWORD
	instr.declare_data_len = len(data)
	assert instr.declare_data_len == len(data)
	for i, d in enumerate(data):
		instr.set_declare_word_value(i, d)
	for i, d in enumerate(data):
		assert instr.get_declare_word_value(i) == d

@pytest.mark.parametrize("data", [
	[0x1234],
	[0x1234, -0x1234],
	[0x1234, -0x1234, 0x4567, -0x4567, 0x1122, -0x1122, 0x3344, -0x3344],
])
def test_dw_i(data):
	assert 1 <= len(data) <= 8
	instr = Instruction()
	instr.code = Code.DECLAREWORD
	instr.declare_data_len = len(data)
	assert instr.declare_data_len == len(data)
	for i, d in enumerate(data):
		instr.set_declare_word_value_i16(i, d)
	for i, d in enumerate(data):
		assert instr.get_declare_word_value_i16(i) == d

@pytest.mark.parametrize("data", [
	[0x1234_5678],
	[0x1234_5678, 0x89AB_CDEF],
	[0x1234_5678, 0x9ABC_CDEF, 0x1122_3344, 0x8899_AABB],
])
def test_dd_u(data):
	assert 1 <= len(data) <= 4
	instr = Instruction()
	instr.code = Code.DECLAREDWORD
	instr.declare_data_len = len(data)
	assert instr.declare_data_len == len(data)
	for i, d in enumerate(data):
		instr.set_declare_dword_value(i, d)
	for i, d in enumerate(data):
		assert instr.get_declare_dword_value(i) == d

@pytest.mark.parametrize("data", [
	[0x1234_5678],
	[0x1234_5678, -0x1234_5678],
	[0x1234_5678, -0x1234_5678, 0x1122_3344, -0x1122_3344],
])
def test_dd_i(data):
	assert 1 <= len(data) <= 4
	instr = Instruction()
	instr.code = Code.DECLAREDWORD
	instr.declare_data_len = len(data)
	assert instr.declare_data_len == len(data)
	for i, d in enumerate(data):
		instr.set_declare_dword_value_i32(i, d)
	for i, d in enumerate(data):
		assert instr.get_declare_dword_value_i32(i) == d

@pytest.mark.parametrize("data", [
	[0x1234_5678_9ABC_DEF0],
	[0x1234_5678_9ABC_DEF0, 0xABCD_EF01_2345_6789],
])
def test_dq_u(data):
	assert 1 <= len(data) <= 2
	instr = Instruction()
	instr.code = Code.DECLAREQWORD
	instr.declare_data_len = len(data)
	assert instr.declare_data_len == len(data)
	for i, d in enumerate(data):
		instr.set_declare_qword_value(i, d)
	for i, d in enumerate(data):
		assert instr.get_declare_qword_value(i) == d

@pytest.mark.parametrize("data", [
	[0x1234_5678_9ABC_DEF0],
	[0x1234_5678_9ABC_DEF0, -0x1234_5678_9ABC_DEF0],
])
def test_dq_i(data):
	assert 1 <= len(data) <= 2
	instr = Instruction()
	instr.code = Code.DECLAREQWORD
	instr.declare_data_len = len(data)
	assert instr.declare_data_len == len(data)
	for i, d in enumerate(data):
		instr.set_declare_qword_value_i64(i, d)
	for i, d in enumerate(data):
		assert instr.get_declare_qword_value_i64(i) == d

@pytest.mark.parametrize("bitness, data, vsib", [
	(64, b"\x29\x18", 0),
	(64, b"\xC4\xE2\x49\x90\x54\xA1\x01", 32),
	(64, b"\xC4\xE2\x49\x91\x54\xA1\x01", 64),
])
def test_vsib(bitness, data, vsib):
	instr = Decoder(bitness, data).decode()
	if vsib == 0:
		assert instr.vsib is None
		assert not instr.is_vsib
		assert not instr.is_vsib32
		assert not instr.is_vsib64
	elif vsib == 32:
		assert instr.vsib is False
		assert instr.is_vsib
		assert instr.is_vsib32
		assert not instr.is_vsib64
	elif vsib == 64:
		assert instr.vsib is True
		assert instr.is_vsib
		assert not instr.is_vsib32
		assert instr.is_vsib64
	else:
		raise ValueError(f"Invalid vsib value: {vsib}")

def test_ip_rel_addr():
	decoder = Decoder(64, b"\x00\x00" b"\x01\x35\x34\x12\x5A\xA5" b"\x67\x01\x35\x34\x12\x5A\xA5", ip=0x1234_5678_9ABC_DEF0)

	instr = decoder.decode()
	assert not instr.is_ip_rel_memory_operand

	instr = decoder.decode()
	assert instr.is_ip_rel_memory_operand
	assert instr.ip_rel_memory_address == 0x1234_5678_4016_F12C

	instr = decoder.decode()
	assert instr.is_ip_rel_memory_operand
	assert instr.ip_rel_memory_address == 0x4016_F133

def test_sp_inc():
	decoder = Decoder(64, b"\x90\x56\x5E")

	instr = decoder.decode()
	assert not instr.is_stack_instruction
	assert instr.stack_pointer_increment == 0

	instr = decoder.decode()
	assert instr.is_stack_instruction
	assert instr.stack_pointer_increment == -8

	instr = decoder.decode()
	assert instr.is_stack_instruction
	assert instr.stack_pointer_increment == 8

@pytest.mark.parametrize("bitness, data, encoding", [
	(64, b"\x56", EncodingKind.LEGACY),
	(64, b"\xC5\xF8\x10\x10", EncodingKind.VEX),
	(64, b"\x62\xF1\x7C\x08\x10\x50\x01", EncodingKind.EVEX),
	(64, b"\x8F\xE8\x48\x85\x10\x40", EncodingKind.XOP),
	(64, b"\x0F\x0F\x88\x34\x12\x5A\xA5\x0C", EncodingKind.D3NOW),
])
def test_encoding(bitness, data, encoding):
	instr = Decoder(bitness, data).decode()
	assert instr.encoding == encoding

def test_cpuid_features():
	instr = Decoder(64, b"\x62\xF1\x7C\x08\x10\x50\x01").decode()
	cpuid_features = instr.cpuid_features()
	assert type(cpuid_features) == list
	assert len(cpuid_features) == 2
	assert set(cpuid_features) == set([CpuidFeature.AVX512VL, CpuidFeature.AVX512F])

def test_cflow():
	decoder = Decoder(64, b"\x90\xCC\x70\x00")

	assert decoder.decode().flow_control == FlowControl.NEXT
	assert decoder.decode().flow_control == FlowControl.INTERRUPT
	assert decoder.decode().flow_control == FlowControl.CONDITIONAL_BRANCH

def test_rflags():
	decoder = Decoder(64, b"\x33\xC0")
	instr = decoder.decode()

	assert instr.rflags_read == RflagsBits.NONE
	assert instr.rflags_written == RflagsBits.NONE
	assert instr.rflags_cleared == RflagsBits.OF | RflagsBits.SF | RflagsBits.CF
	assert instr.rflags_set == RflagsBits.ZF | RflagsBits.PF
	assert instr.rflags_undefined == RflagsBits.AF
	assert instr.rflags_modified == RflagsBits.OF | RflagsBits.SF | RflagsBits.ZF | RflagsBits.AF | RflagsBits.CF | RflagsBits.PF

def test_br_checks():
	instr = Decoder(64, b"\x70\x00").decode()
	assert instr.is_jcc_short_or_near
	assert instr.is_jcc_short
	assert not instr.is_jcc_near

	instr = Decoder(64, b"\x0F\x80\x00\x00\x00\x00").decode()
	assert instr.is_jcc_short_or_near
	assert not instr.is_jcc_short
	assert instr.is_jcc_near

	instr = Decoder(64, b"\xEB\x00").decode()
	assert instr.is_jmp_short_or_near
	assert instr.is_jmp_short
	assert not instr.is_jmp_near

	instr = Decoder(64, b"\xE9\x00\x00\x00\x00").decode()
	assert instr.is_jmp_short_or_near
	assert not instr.is_jmp_short
	assert instr.is_jmp_near

	instr = Decoder(32, b"\xEA\x00\x00\x00\x00\x00\x00").decode()
	assert instr.is_jmp_far
	assert not instr.is_call_far
	assert not instr.is_jmp_far_indirect
	assert not instr.is_call_far_indirect

	instr = Decoder(32, b"\x9A\x00\x00\x00\x00\x00\x00").decode()
	assert not instr.is_jmp_far
	assert instr.is_call_far
	assert not instr.is_jmp_far_indirect
	assert not instr.is_call_far_indirect

	instr = Decoder(64, b"\x48\xFF\x28").decode()
	assert not instr.is_jmp_far
	assert not instr.is_call_far
	assert instr.is_jmp_far_indirect
	assert not instr.is_call_far_indirect

	instr = Decoder(64, b"\x48\xFF\x18").decode()
	assert not instr.is_jmp_far
	assert not instr.is_call_far
	assert not instr.is_jmp_far_indirect
	assert instr.is_call_far_indirect

	instr = Decoder(64, b"\xE8\x00\x00\x00\x00").decode()
	assert instr.is_call_near
	assert not instr.is_jmp_near_indirect
	assert not instr.is_call_near_indirect

	instr = Decoder(64, b"\xFF\x20").decode()
	assert not instr.is_call_near
	assert instr.is_jmp_near_indirect
	assert not instr.is_call_near_indirect

	instr = Decoder(64, b"\xFF\x10").decode()
	assert not instr.is_call_near
	assert not instr.is_jmp_near_indirect
	assert instr.is_call_near_indirect

def test_condition_code():
	instr = Decoder(64, b"\x70\x00").decode()

	assert instr.condition_code == ConditionCode.O
	instr.negate_condition_code()
	assert instr.condition_code == ConditionCode.NO

def test_short_near_br():
	instr = Decoder(64, b"\x70\x00").decode()

	assert instr.code == Code.JO_REL8_64
	instr.as_short_branch()
	assert instr.code == Code.JO_REL8_64
	instr.as_near_branch()
	assert instr.code == Code.JO_REL32_64
	instr.as_near_branch()
	assert instr.code == Code.JO_REL32_64

def test_op_code():
	instr = Decoder(64, b"\x70\x00").decode()
	idef1 = instr.op_code()
	assert idef1.code == Code.JO_REL8_64
	assert idef1 == OpCodeInfo(Code.JO_REL8_64)

def test_repr_str():
	instr = Decoder(64, b"\x48\x05\xA5\x5A\x34\x82").decode()
	assert repr(instr) == "add rax,0FFFFFFFF82345AA5h"
	assert str(instr) == "add rax,0FFFFFFFF82345AA5h"

def test_format():
	decoder = Decoder(64, b"\x48\x05\xA5\x5A\x34\x82" b"\x48\x8B\x05\x88\xA9\xCB\xED" b"\x70\x00", ip=0x1234_5678_9ABC_DEF0)

	instr = decoder.decode()
	assert f"{instr}" == "add rax,0FFFFFFFF82345AA5h"
	assert f"{instr:f}" == "add rax,0FFFFFFFF82345AA5h"
	assert f"{instr:g}" == "add $0xFFFFFFFF82345AA5,%rax"
	assert f"{instr:i}" == "add rax,0FFFFFFFF82345AA5h"
	assert f"{instr:m}" == "add rax,0FFFFFFFF82345AA5h"
	assert f"{instr:n}" == "add rax,0FFFFFFFF82345AA5h"
	assert f"{instr:x}" == "add rax,0xffffffff82345aa5"
	assert f"{instr:X}" == "add rax,0xFFFFFFFF82345AA5"
	assert f"{instr:h}" == "add rax,0ffffffff82345aa5h"
	assert f"{instr:H}" == "add rax,0FFFFFFFF82345AA5h"
	assert f"{instr:Uh}" == "ADD RAX,0ffffffff82345aa5h"
	assert f"{instr:s}" == "add rax, 0FFFFFFFF82345AA5h"
	assert f"{instr:gG}" == "addq $0xFFFFFFFF82345AA5,%rax"
	assert f"{instr:_}" == "add rax,0_FFFF_FFFF_8234_5AA5h"
	assert f"{instr:ixUs_}" == "ADD RAX, 0xffff_ffff_8234_5aa5"

	instr = decoder.decode()
	assert f"{instr}" == "mov rax,[1234567888888885h]"
	assert f"{instr:r}" == "mov rax,[rip-12345678h]"
	assert f"{instr:S}" == "mov rax,ds:[1234567888888885h]"
	assert f"{instr:M}" == "mov rax,qword ptr [1234567888888885h]"

	instr = decoder.decode()
	assert f"{instr}" == "jo short 123456789ABCDEFFh"
	assert f"{instr:B}" == "jo 123456789ABCDEFFh"

def test_format_raise():
	instr = Decoder(64, b"\x48\x05\xA5\x5A\x34\x82").decode()
	with pytest.raises(ValueError):
		f"{instr:!}"

def test_op_kind_raise():
	instr = Instruction()
	with pytest.raises(ValueError):
		instr.op_kind(100)
	with pytest.raises(ValueError):
		instr.set_op_kind(100, OpKind.REGISTER)
	with pytest.raises(ValueError):
		instr.set_op_kind(0, 12345)

def test_op_register_raise():
	instr = Instruction()
	with pytest.raises(ValueError):
		instr.op_register(100)
	with pytest.raises(ValueError):
		instr.set_op_register(100, Register.RAX)
	with pytest.raises(ValueError):
		instr.set_op_register(0, 12345)

def test_immediate_raise():
	instr = Instruction()

	not_imm_op_kinds = [
		OpKind.REGISTER,
		OpKind.NEAR_BRANCH16,
		OpKind.NEAR_BRANCH32,
		OpKind.NEAR_BRANCH64,
		OpKind.FAR_BRANCH16,
		OpKind.FAR_BRANCH32,
		OpKind.MEMORY_SEG_SI,
		OpKind.MEMORY_SEG_ESI,
		OpKind.MEMORY_SEG_RSI,
		OpKind.MEMORY_SEG_DI,
		OpKind.MEMORY_SEG_EDI,
		OpKind.MEMORY_SEG_RDI,
		OpKind.MEMORY_ESDI,
		OpKind.MEMORY_ESEDI,
		OpKind.MEMORY_ESRDI,
		OpKind.MEMORY,
	]

	imm_op_kinds = [
		OpKind.IMMEDIATE8,
		OpKind.IMMEDIATE8_2ND,
		OpKind.IMMEDIATE16,
		OpKind.IMMEDIATE32,
		OpKind.IMMEDIATE64,
		OpKind.IMMEDIATE8TO16,
		OpKind.IMMEDIATE8TO32,
		OpKind.IMMEDIATE8TO64,
		OpKind.IMMEDIATE32TO64,
	]

	for kind in not_imm_op_kinds:
		instr.op0_kind = kind
		with pytest.raises(ValueError):
			instr.immediate(0)
		with pytest.raises(ValueError):
			instr.set_immediate_i32(0, 0)
		with pytest.raises(ValueError):
			instr.set_immediate_u32(0, 0)
		with pytest.raises(ValueError):
			instr.set_immediate_i64(0, 0)
		with pytest.raises(ValueError):
			instr.set_immediate_u64(0, 0)
	for kind in imm_op_kinds:
		instr.op0_kind = kind
		instr.immediate(0)
		instr.set_immediate_i32(0, 0)
		instr.set_immediate_u32(0, 0)
		instr.set_immediate_i64(0, 0)
		instr.set_immediate_u64(0, 0)

	instr.op0_kind = OpKind.IMMEDIATE8
	instr.immediate(0)
	with pytest.raises(ValueError):
		instr.immediate(100)
	with pytest.raises(ValueError):
		instr.set_immediate_i32(100, 0)
	with pytest.raises(ValueError):
		instr.set_immediate_u32(100, 0)
	with pytest.raises(ValueError):
		instr.set_immediate_i64(100, 0)
	with pytest.raises(ValueError):
		instr.set_immediate_u64(100, 0)

def test_db_raise():
	instr = Instruction()
	instr.code = Code.DECLAREBYTE
	NUM = 16
	instr.set_declare_byte_value_i8(NUM - 1, 0)
	instr.set_declare_byte_value(NUM - 1, 0)
	instr.get_declare_byte_value(NUM - 1)
	instr.get_declare_byte_value_i8(NUM - 1)
	with pytest.raises(ValueError):
		instr.set_declare_byte_value_i8(NUM, 0)
	with pytest.raises(ValueError):
		instr.set_declare_byte_value(NUM, 0)
	with pytest.raises(ValueError):
		instr.get_declare_byte_value(NUM)
	with pytest.raises(ValueError):
		instr.get_declare_byte_value_i8(NUM)

def test_dw_raise():
	instr = Instruction()
	instr.code = Code.DECLAREWORD
	NUM = 8
	instr.set_declare_word_value_i16(NUM - 1, 0)
	instr.set_declare_word_value(NUM - 1, 0)
	instr.get_declare_word_value(NUM - 1)
	instr.get_declare_word_value_i16(NUM - 1)
	with pytest.raises(ValueError):
		instr.set_declare_word_value_i16(NUM, 0)
	with pytest.raises(ValueError):
		instr.set_declare_word_value(NUM, 0)
	with pytest.raises(ValueError):
		instr.get_declare_word_value(NUM)
	with pytest.raises(ValueError):
		instr.get_declare_word_value_i16(NUM)

def test_dd_raise():
	instr = Instruction()
	instr.code = Code.DECLAREDWORD
	NUM = 4
	instr.set_declare_dword_value_i32(NUM - 1, 0)
	instr.set_declare_dword_value(NUM - 1, 0)
	instr.get_declare_dword_value(NUM - 1)
	instr.get_declare_dword_value_i32(NUM - 1)
	with pytest.raises(ValueError):
		instr.set_declare_dword_value_i32(NUM, 0)
	with pytest.raises(ValueError):
		instr.set_declare_dword_value(NUM, 0)
	with pytest.raises(ValueError):
		instr.get_declare_dword_value(NUM)
	with pytest.raises(ValueError):
		instr.get_declare_dword_value_i32(NUM)

def test_dq_raise():
	instr = Instruction()
	instr.code = Code.DECLAREQWORD
	NUM = 2
	instr.set_declare_qword_value_i64(NUM - 1, 0)
	instr.set_declare_qword_value(NUM - 1, 0)
	instr.get_declare_qword_value(NUM - 1)
	instr.get_declare_qword_value_i64(NUM - 1)
	with pytest.raises(ValueError):
		instr.set_declare_qword_value_i64(NUM, 0)
	with pytest.raises(ValueError):
		instr.set_declare_qword_value(NUM, 0)
	with pytest.raises(ValueError):
		instr.get_declare_qword_value(NUM)
	with pytest.raises(ValueError):
		instr.get_declare_qword_value_i64(NUM)

def test_code_size_raise():
	instr = Instruction()
	instr.code_size = CodeSize.CODE64
	with pytest.raises(ValueError):
		instr.code_size = 1234

def test_code_raise():
	instr = Instruction()
	instr.code = Code.EVEX_VAESENCLAST_ZMM_ZMM_ZMMM512
	with pytest.raises(ValueError):
		instr.code = 10000

def test_segment_prefix_raise():
	instr = Instruction()
	instr.segment_prefix = Register.FS
	with pytest.raises(ValueError):
		instr.segment_prefix = 1234

def test_memory_base_raise():
	instr = Instruction()
	instr.memory_base = Register.RAX
	with pytest.raises(ValueError):
		instr.memory_base = 1234

def test_memory_index_raise():
	instr = Instruction()
	instr.memory_index = Register.RAX
	with pytest.raises(ValueError):
		instr.memory_index = 1234

def test_op_register_raise():
	instr = Instruction()

	instr.op0_register = Register.RAX
	with pytest.raises(ValueError):
		instr.op0_register = 1234

	instr.set_op_register(0, Register.RAX)
	with pytest.raises(ValueError):
		instr.set_op_register(0, 1234)

	instr.op1_register = Register.RAX
	with pytest.raises(ValueError):
		instr.op1_register = 1234

	instr.set_op_register(1, Register.RAX)
	with pytest.raises(ValueError):
		instr.set_op_register(1, 1234)

	instr.op2_register = Register.RAX
	with pytest.raises(ValueError):
		instr.op2_register = 1234

	instr.set_op_register(2, Register.RAX)
	with pytest.raises(ValueError):
		instr.set_op_register(2, 1234)

	instr.op3_register = Register.RAX
	with pytest.raises(ValueError):
		instr.op3_register = 1234

	instr.set_op_register(3, Register.RAX)
	with pytest.raises(ValueError):
		instr.set_op_register(3, 1234)

	instr.op4_register = Register.NONE
	with pytest.raises(ValueError):
		instr.op4_register = 1234

	instr.set_op_register(4, Register.NONE)
	with pytest.raises(ValueError):
		instr.set_op_register(4, 1234)

def test_op_mask_raise():
	instr = Instruction()
	instr.op_mask = Register.K1
	with pytest.raises(ValueError):
		instr.op_mask = 1234

def test_fpu_stack_increment_info():
	instr = Decoder(64, b"\xDA\x18").decode()
	info = instr.fpu_stack_increment_info()
	assert info.increment == 1
	assert not info.conditional
	assert info.writes_top



# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

import pytest
from iced_x86 import *

@pytest.mark.parametrize("bitness", [16, 32, 64, 0, 15, 128])
def test_invalid_bitness(bitness):
	if bitness == 16 or bitness == 32 or bitness == 64:
		BlockEncoder(bitness)
	else:
		with pytest.raises(ValueError):
			BlockEncoder(bitness)

@pytest.mark.parametrize("fix_branches", [False, True])
def test_fix_branches_arg(fix_branches):
	decoder = Decoder(64, b"\x72\x00", ip=0x1234_5678_9ABC_DEF0)
	new_rip = 0xFEDC_BA98_7654_3210
	instr = decoder.decode()
	encoder = BlockEncoder(64, fix_branches)
	encoder.add(instr)
	if fix_branches:
		encoder.encode(new_rip)
	else:
		with pytest.raises(ValueError):
			encoder.encode(new_rip)

def test_fix_branches_defaults_to_true():
	decoder = Decoder(64, b"\x72\x00", ip=0x1234_5678_9ABC_DEF0)
	new_rip = 0xFEDC_BA98_7654_3210
	instr = decoder.decode()
	encoder = BlockEncoder(64)
	encoder.add(instr)
	# No exception
	encoded_data = encoder.encode(new_rip)
	assert len(encoded_data) > 2

def test_encode():
	decoder = Decoder(64, b"\xF3\x90\x90\x48\x09\xCE\x48\x09\xCE\x90\xF3\x90\x48\x09\xCE", ip=0x1234_5678_9ABC_DEF0)
	new_rip = 0xFEDC_BA98_7654_3210
	instrs = [instr for instr in decoder]
	encoder = BlockEncoder(64)
	encoder.add(instrs[0])
	encoder.add(instrs[1])
	encoder.add(instrs[2])
	encoder.add_many(instrs[3:6])
	encoder.add(instrs[6])
	encoded_data = encoder.encode(new_rip)
	assert type(encoded_data) == bytes
	assert encoded_data == b"\xF3\x90\x90\x48\x09\xCE\x48\x09\xCE\x90\xF3\x90\x48\x09\xCE"

def test_encode_invalid():
	encoder = BlockEncoder(64)
	encoder.add(Instruction())
	with pytest.raises(ValueError):
		encoder.encode(0)

def test_encode_empty():
	encoder = BlockEncoder(64)
	encoded_data = encoder.encode(0xFEDC_BA98_7654_3210)
	assert type(encoded_data) == bytes
	assert encoded_data == b""



# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

import pytest
from iced_x86 import *

def test_props():
	idef = OpCodeInfo(Code.EVEX_VMOVAPD_YMM_K1Z_YMMM256)

	assert idef.code == Code.EVEX_VMOVAPD_YMM_K1Z_YMMM256
	assert idef.mnemonic == Mnemonic.VMOVAPD
	assert idef.encoding == EncodingKind.EVEX
	assert idef.is_instruction
	assert idef.mode16
	assert idef.mode32
	assert idef.mode64
	assert not idef.fwait
	assert idef.operand_size == 0
	assert idef.address_size == 0
	assert idef.l == 1
	assert idef.w == 1
	assert not idef.is_lig
	assert not idef.is_wig
	assert not idef.is_wig32
	assert idef.tuple_type == TupleType.N32
	assert idef.memory_size == MemorySize.PACKED256_FLOAT64
	assert idef.broadcast_memory_size == MemorySize.UNKNOWN
	assert not idef.can_broadcast
	assert not idef.can_use_rounding_control
	assert not idef.can_suppress_all_exceptions
	assert idef.can_use_op_mask_register
	assert not idef.require_op_mask_register
	assert idef.can_use_zeroing_masking
	assert not idef.can_use_lock_prefix
	assert not idef.can_use_xacquire_prefix
	assert not idef.can_use_xrelease_prefix
	assert not idef.can_use_rep_prefix
	assert not idef.can_use_repne_prefix
	assert not idef.can_use_bnd_prefix
	assert not idef.can_use_hint_taken_prefix
	assert not idef.can_use_notrack_prefix
	assert not idef.ignores_rounding_control
	assert not idef.amd_lock_reg_bit
	assert not idef.default_op_size64
	assert not idef.force_op_size64
	assert not idef.intel_force_op_size64
	assert not idef.must_be_cpl0
	assert idef.cpl0
	assert idef.cpl1
	assert idef.cpl2
	assert idef.cpl3
	assert not idef.is_input_output
	assert not idef.is_nop
	assert not idef.is_reserved_nop
	assert not idef.is_serializing_intel
	assert not idef.is_serializing_amd
	assert not idef.may_require_cpl0
	assert not idef.is_cet_tracked
	assert not idef.is_non_temporal
	assert not idef.is_fpu_no_wait
	assert not idef.ignores_mod_bits
	assert not idef.no66
	assert not idef.nfx
	assert not idef.requires_unique_reg_nums
	assert not idef.is_privileged
	assert not idef.is_save_restore
	assert not idef.is_stack_instruction
	assert not idef.ignores_segment
	assert not idef.is_op_mask_read_write
	assert not idef.real_mode
	assert idef.protected_mode
	assert not idef.virtual8086_mode
	assert idef.compatibility_mode
	assert idef.long_mode
	assert idef.use_outside_smm
	assert idef.use_in_smm
	assert idef.use_outside_enclave_sgx
	assert idef.use_in_enclave_sgx1
	assert idef.use_in_enclave_sgx2
	assert idef.use_outside_vmx_op
	assert idef.use_in_vmx_root_op
	assert idef.use_in_vmx_non_root_op
	assert idef.use_outside_seam
	assert idef.use_in_seam
	assert not idef.tdx_non_root_gen_ud
	assert not idef.tdx_non_root_gen_ve
	assert not idef.tdx_non_root_may_gen_ex
	assert not idef.intel_vm_exit
	assert not idef.intel_may_vm_exit
	assert not idef.intel_smm_vm_exit
	assert not idef.amd_vm_exit
	assert not idef.amd_may_vm_exit
	assert not idef.tsx_abort
	assert not idef.tsx_impl_abort
	assert not idef.tsx_may_abort
	assert idef.intel_decoder16
	assert idef.intel_decoder32
	assert idef.intel_decoder64
	assert idef.amd_decoder16
	assert idef.amd_decoder32
	assert idef.amd_decoder64
	assert idef.decoder_option == DecoderOptions.NONE
	assert idef.table == OpCodeTableKind.T0F
	assert idef.mandatory_prefix == MandatoryPrefix.P66
	assert idef.op_code == 0x28
	assert idef.op_code_len == 1
	assert not idef.is_group
	assert idef.group_index == -1
	assert not idef.is_rm_group
	assert idef.rm_group_index == -1
	assert idef.op_count == 2
	assert idef.op0_kind == OpCodeOperandKind.YMM_REG
	assert idef.op1_kind == OpCodeOperandKind.YMM_OR_MEM
	assert idef.op2_kind == OpCodeOperandKind.NONE
	assert idef.op3_kind == OpCodeOperandKind.NONE
	assert idef.op4_kind == OpCodeOperandKind.NONE
	assert idef.op_kind(0) == idef.op0_kind
	assert idef.op_kind(1) == idef.op1_kind
	assert idef.op_kind(2) == idef.op2_kind
	assert idef.op_kind(3) == idef.op3_kind
	assert idef.op_kind(4) == idef.op4_kind
	assert type(idef.op_kinds()) == list
	assert idef.op_kinds() == [OpCodeOperandKind.YMM_REG, OpCodeOperandKind.YMM_OR_MEM]
	assert idef.is_available_in_mode(16)
	assert idef.is_available_in_mode(32)
	assert idef.is_available_in_mode(64)
	assert idef.op_code_string == "EVEX.256.66.0F.W1 28 /r"
	assert idef.instruction_string == "VMOVAPD ymm1 {k1}{z}, ymm2/m256"
	assert repr(idef) == "VMOVAPD ymm1 {k1}{z}, ymm2/m256"
	assert str(idef) == "VMOVAPD ymm1 {k1}{z}, ymm2/m256"
	assert f"{idef}" == "VMOVAPD ymm1 {k1}{z}, ymm2/m256"
	assert f"{idef:}" == "VMOVAPD ymm1 {k1}{z}, ymm2/m256"
	assert f"{idef:i}" == "VMOVAPD ymm1 {k1}{z}, ymm2/m256"
	assert f"{idef:o}" == "EVEX.256.66.0F.W1 28 /r"

def test_eq_ne_hash():
	idef1 = OpCodeInfo(Code.EVEX_VMOVAPD_YMM_K1Z_YMMM256)
	idef2 = OpCodeInfo(Code.EVEX_VMOVAPD_YMM_K1Z_YMMM256)
	idef3 = OpCodeInfo(Code.EVEX_VMOVAPD_XMM_K1Z_XMMM128)

	assert idef1 == idef1
	assert idef1 == idef2
	assert idef1 != idef3

	assert not (idef1 != idef1)
	assert not (idef1 != idef2)
	assert not (idef1 == idef3)

	assert idef1 != 1
	assert idef1 != 1.23
	assert idef1 != None
	assert idef1 != []
	assert idef1 != {}
	assert idef1 != (1, 2)

	assert not (idef1 == 1)
	assert not (idef1 == 1.23)
	assert not (idef1 == None)
	assert not (idef1 == [])
	assert not (idef1 == {})
	assert not (idef1 == (1, 2))

	assert hash(idef1) == hash(idef1)
	assert hash(idef1) == hash(idef2)

def test_invalid_format_spec():
	idef = OpCodeInfo(Code.EVEX_VMOVAPD_YMM_K1Z_YMMM256)
	with pytest.raises(ValueError):
		f"{idef:Q}"

def test_invalid_op_kind_arg():
	idef = OpCodeInfo(Code.EVEX_VMOVAPD_YMM_K1Z_YMMM256)
	with pytest.raises(ValueError):
		idef.op_kind(100)

@pytest.mark.parametrize("bitness", [16, 32, 64, 0, 15, 128])
def test_invalid_bitness(bitness):
	idef = OpCodeInfo(Code.EVEX_VMOVAPD_YMM_K1Z_YMMM256)
	idef.is_available_in_mode(bitness)

def test_op_code_raise():
	with pytest.raises(ValueError):
		OpCodeInfo(10000)



# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Mnemonic condition code selector (eg. ``JL`` / ``JNGE``)
"""

L: int = 0
"""
``JL``, ``CMOVL``, ``SETL``
"""
NGE: int = 1
"""
``JNGE``, ``CMOVNGE``, ``SETNGE``
"""



# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Memory size options used by the formatters
"""

DEFAULT: int = 0
"""
Show memory size if the assembler requires it, else don't show anything
"""
ALWAYS: int = 1
"""
Always show the memory size, even if the assembler doesn't need it
"""
MINIMAL: int = 2
"""
Show memory size if a human can't figure out the size of the operand
"""
NEVER: int = 3
"""
Never show memory size
"""



# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Instruction condition code (used by ``Jcc``, ``SETcc``, ``CMOVcc``, ``LOOPcc``)
"""

NONE: int = 0
"""
The instruction doesn't have a condition code
"""
O: int = 1
"""
Overflow (``OF=1``)
"""
NO: int = 2
"""
Not overflow (``OF=0``)
"""
B: int = 3
"""
Below (unsigned) (``CF=1``)
"""
AE: int = 4
"""
Above or equal (unsigned) (``CF=0``)
"""
E: int = 5
"""
Equal / zero (``ZF=1``)
"""
NE: int = 6
"""
Not equal / zero (``ZF=0``)
"""
BE: int = 7
"""
Below or equal (unsigned) (``CF=1 or ZF=1``)
"""
A: int = 8
"""
Above (unsigned) (``CF=0 and ZF=0``)
"""
S: int = 9
"""
Signed (``SF=1``)
"""
NS: int = 10
"""
Not signed (``SF=0``)
"""
P: int = 11
"""
Parity (``PF=1``)
"""
NP: int = 12
"""
Not parity (``PF=0``)
"""
L: int = 13
"""
Less (signed) (``SF!=OF``)
"""
GE: int = 14
"""
Greater than or equal (signed) (``SF=OF``)
"""
LE: int = 15
"""
Less than or equal (signed) (``ZF=1 or SF!=OF``)
"""
G: int = 16
"""
Greater (signed) (``ZF=0 and SF=OF``)
"""



# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Formatter syntax (GNU Assembler, Intel XED, masm, nasm)
"""

GAS: int = 0
"""
GNU Assembler (AT&T)
"""
INTEL: int = 1
"""
Intel XED
"""
MASM: int = 2
"""
masm
"""
NASM: int = 3
"""
nasm
"""



# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

import pytest
from iced_x86 import *

@pytest.mark.parametrize("bitness", [16, 32, 64, 0, 15, 128])
def test_invalid_bitness(bitness):
	if bitness == 16 or bitness == 32 or bitness == 64:
		Encoder(bitness)
	else:
		with pytest.raises(ValueError):
			Encoder(bitness)

@pytest.mark.parametrize("capacity", [0, 1, 0x1234])
def test_capacity_arg(capacity):
	Encoder(64, capacity)

@pytest.mark.parametrize("bitness", [16, 32, 64])
def test_bitness(bitness):
	encoder = Encoder(bitness)
	assert encoder.bitness == bitness

@pytest.mark.parametrize("bitness", [16, 32, 64])
def test_options(bitness):
	encoder = Encoder(bitness)
	assert not encoder.prevent_vex2
	assert encoder.vex_wig == 0
	assert encoder.vex_lig == 0
	assert encoder.evex_wig == 0
	assert encoder.evex_lig == 0

	encoder.prevent_vex2 = True
	encoder.vex_wig = 1
	encoder.vex_lig = 1
	encoder.evex_wig = 1
	encoder.evex_lig = 2

	assert encoder.prevent_vex2
	assert encoder.vex_wig == 1
	assert encoder.vex_lig == 1
	assert encoder.evex_wig == 1
	assert encoder.evex_lig == 2

@pytest.mark.parametrize("bitness, data", [
	(16, b"\x03\xCE\x90\xF3\x90"),
	(32, b"\x03\xCE\x90\xF3\x90"),
	(64, b"\x48\x09\xCE\x90\xF3\x90"),
])
def test_encode(bitness, data):
	encoder = Encoder(bitness)
	decoder = Decoder(bitness, data, ip=0x1234_5678_9ABC_DEF0)
	rip = decoder.ip
	for instr in decoder:
		instr_len = encoder.encode(instr, rip)
		assert instr.len == instr_len
		rip += instr_len
	encoded_data = encoder.take_buffer()
	assert type(encoded_data) == bytes
	assert data == encoded_data

def test_encode_rip_63_set():
	decoder = Decoder(64, b"\x48\x09\xCE\x90\xF3\x90")
	instr = decoder.decode()
	encoder = Encoder(64)
	assert encoder.encode(instr, 0x1234_5678_9ABC_DEF0) == instr.len
	assert encoder.encode(instr, 0xFEDC_BA98_7654_3210) == instr.len

def test_write_u8():
	decoder = Decoder(64, b"\x48\x09\xCE\x90\xF3\x90")
	instr = decoder.decode()
	encoder = Encoder(64)
	encoder.write_u8(0x12)
	encoder.write_u8(0x34)
	assert encoder.encode(instr, 0) == 3
	encoder.write_u8(0x56)
	encoder.write_u8(0x78)
	encoder.write_u8(0x9A)
	encoded_data = encoder.take_buffer()
	assert encoded_data == b"\x12\x34\x48\x09\xCE\x56\x78\x9A"

def test_encode_invalid_instruction():
	# Can't encode INVALID
	instr = Instruction()
	encoder = Encoder(64)
	with pytest.raises(ValueError):
		encoder.encode(instr, instr.ip)

	# Jcc SHORT with a target too far away
	decoder = Decoder(64, b"\x72\x00", ip=0x1234_5678_9ABC_DEF0)
	decoder.decode_out(instr)
	encoder.encode(instr, instr.ip)
	with pytest.raises(ValueError):
		encoder.encode(instr, 0xFEDC_BA98_7654_3210)

def test_offsets():
	decoder = Decoder(64, b"\x90\x83\xB3\x34\x12\x5A\xA5\x5A")
	encoder = Encoder(64)

	instr = decoder.decode()
	assert encoder.encode(instr, instr.ip) == 1
	co = encoder.get_constant_offsets()
	assert co.displacement_offset == 0
	assert co.displacement_size == 0
	assert co.immediate_offset == 0
	assert co.immediate_size == 0
	assert co.immediate_offset2 == 0
	assert co.immediate_size2 == 0
	assert not co.has_displacement
	assert not co.has_immediate
	assert not co.has_immediate2

	instr = decoder.decode()
	assert encoder.encode(instr, instr.ip) == 7
	co = encoder.get_constant_offsets()
	assert co.displacement_offset == 2
	assert co.displacement_size == 4
	assert co.immediate_offset == 6
	assert co.immediate_size == 1
	assert co.immediate_offset2 == 0
	assert co.immediate_size2 == 0
	assert co.has_displacement
	assert co.has_immediate
	assert not co.has_immediate2



# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

import pytest
from iced_x86 import *

def test_register_ext():
	assert RegisterExt.base(Register.DL) == Register.AL
	assert RegisterExt.base(Register.R8W) == Register.AX
	assert RegisterExt.base(Register.R15D) == Register.EAX
	assert RegisterExt.base(Register.R13) == Register.RAX
	assert RegisterExt.base(Register.FS) == Register.ES
	assert RegisterExt.base(Register.XMM2) == Register.XMM0
	assert RegisterExt.base(Register.YMM20) == Register.YMM0
	assert RegisterExt.base(Register.ZMM31) == Register.ZMM0

	assert RegisterExt.number(Register.DL) == 2
	assert RegisterExt.number(Register.R15) == 15
	assert RegisterExt.number(Register.YMM21) == 21

	assert RegisterExt.full_register(Register.CL) == Register.RCX
	assert RegisterExt.full_register(Register.DX) == Register.RDX
	assert RegisterExt.full_register(Register.EBX) == Register.RBX
	assert RegisterExt.full_register(Register.RSP) == Register.RSP
	assert RegisterExt.full_register(Register.XMM2) == Register.ZMM2
	assert RegisterExt.full_register(Register.YMM22) == Register.ZMM22
	assert RegisterExt.full_register(Register.ZMM11) == Register.ZMM11

	assert RegisterExt.full_register32(Register.CL) == Register.ECX
	assert RegisterExt.full_register32(Register.DX) == Register.EDX
	assert RegisterExt.full_register32(Register.EBX) == Register.EBX
	assert RegisterExt.full_register32(Register.RSP) == Register.ESP
	assert RegisterExt.full_register32(Register.XMM2) == Register.ZMM2
	assert RegisterExt.full_register32(Register.YMM22) == Register.ZMM22
	assert RegisterExt.full_register32(Register.ZMM11) == Register.ZMM11

	assert RegisterExt.size(Register.DL) == 1
	assert RegisterExt.size(Register.R8W) == 2
	assert RegisterExt.size(Register.R15D) == 4
	assert RegisterExt.size(Register.R13) == 8
	assert RegisterExt.size(Register.FS) == 2
	assert RegisterExt.size(Register.XMM2) == 16
	assert RegisterExt.size(Register.YMM20) == 32
	assert RegisterExt.size(Register.ZMM31) == 64

	assert not RegisterExt.is_segment_register(Register.CX)
	assert RegisterExt.is_segment_register(Register.GS)

	assert RegisterExt.is_gpr(Register.CL)
	assert RegisterExt.is_gpr(Register.DX)
	assert RegisterExt.is_gpr(Register.ESP)
	assert RegisterExt.is_gpr(Register.R15)
	assert not RegisterExt.is_gpr(Register.ES)

	assert RegisterExt.is_gpr8(Register.CL)
	assert not RegisterExt.is_gpr8(Register.DX)
	assert not RegisterExt.is_gpr8(Register.ESP)
	assert not RegisterExt.is_gpr8(Register.R15)
	assert not RegisterExt.is_gpr8(Register.ES)

	assert not RegisterExt.is_gpr16(Register.CL)
	assert RegisterExt.is_gpr16(Register.DX)
	assert not RegisterExt.is_gpr16(Register.ESP)
	assert not RegisterExt.is_gpr16(Register.R15)
	assert not RegisterExt.is_gpr16(Register.ES)

	assert not RegisterExt.is_gpr32(Register.CL)
	assert not RegisterExt.is_gpr32(Register.DX)
	assert RegisterExt.is_gpr32(Register.ESP)
	assert not RegisterExt.is_gpr32(Register.R15)
	assert not RegisterExt.is_gpr32(Register.ES)

	assert not RegisterExt.is_gpr64(Register.CL)
	assert not RegisterExt.is_gpr64(Register.DX)
	assert not RegisterExt.is_gpr64(Register.ESP)
	assert RegisterExt.is_gpr64(Register.R15)
	assert not RegisterExt.is_gpr64(Register.ES)

	assert not RegisterExt.is_vector_register(Register.CL)
	assert RegisterExt.is_vector_register(Register.XMM1)
	assert RegisterExt.is_vector_register(Register.YMM2)
	assert RegisterExt.is_vector_register(Register.ZMM3)

	assert not RegisterExt.is_xmm(Register.CL)
	assert RegisterExt.is_xmm(Register.XMM1)
	assert not RegisterExt.is_xmm(Register.YMM2)
	assert not RegisterExt.is_xmm(Register.ZMM3)

	assert not RegisterExt.is_ymm(Register.CL)
	assert not RegisterExt.is_ymm(Register.XMM1)
	assert RegisterExt.is_ymm(Register.YMM2)
	assert not RegisterExt.is_ymm(Register.ZMM3)

	assert not RegisterExt.is_zmm(Register.CL)
	assert not RegisterExt.is_zmm(Register.XMM1)
	assert not RegisterExt.is_zmm(Register.YMM2)
	assert RegisterExt.is_zmm(Register.ZMM3)

	assert not RegisterExt.is_ip(Register.CL)
	assert RegisterExt.is_ip(Register.EIP)
	assert RegisterExt.is_ip(Register.RIP)

	assert not RegisterExt.is_k(Register.CL)
	assert RegisterExt.is_k(Register.K3)

	assert not RegisterExt.is_cr(Register.CL)
	assert RegisterExt.is_cr(Register.CR3)

	assert not RegisterExt.is_dr(Register.CL)
	assert RegisterExt.is_dr(Register.DR3)

	assert not RegisterExt.is_tr(Register.CL)
	assert RegisterExt.is_tr(Register.TR3)

	assert not RegisterExt.is_st(Register.CL)
	assert RegisterExt.is_st(Register.ST3)

	assert not RegisterExt.is_bnd(Register.CL)
	assert RegisterExt.is_bnd(Register.BND3)

	assert not RegisterExt.is_mm(Register.CL)
	assert RegisterExt.is_mm(Register.MM3)

	assert not RegisterExt.is_tmm(Register.CL)
	assert RegisterExt.is_tmm(Register.TMM3)

@pytest.mark.parametrize("create", [
	lambda register: RegisterExt.info(register),
	lambda register: RegisterInfo(register),
])
def test_register_info(create):
	info = create(Register.R10D)
	assert info.register == Register.R10D
	assert info.base == Register.EAX
	assert info.number == 10
	assert info.full_register == Register.R10
	assert info.full_register32 == Register.R10D
	assert info.size == 4

@pytest.mark.parametrize("create", [
	lambda register: RegisterExt.info(register),
	lambda register: RegisterInfo(register),
])
def test_register_invalid_arg(create):
	with pytest.raises(ValueError):
		create(1234)

def test_ext_base_invalid_arg():
	with pytest.raises(ValueError):
		RegisterExt.base(1234)

def test_ext_number_invalid_arg():
	with pytest.raises(ValueError):
		RegisterExt.number(1234)

def test_ext_full_register_invalid_arg():
	with pytest.raises(ValueError):
		RegisterExt.full_register(1234)

def test_ext_full_register32_invalid_arg():
	with pytest.raises(ValueError):
		RegisterExt.full_register32(1234)

def test_ext_size_invalid_arg():
	with pytest.raises(ValueError):
		RegisterExt.size(1234)

def test_ext_is_segment_register_invalid_arg():
	with pytest.raises(ValueError):
		RegisterExt.is_segment_register(1234)

def test_ext_is_gpr_invalid_arg():
	with pytest.raises(ValueError):
		RegisterExt.is_gpr(1234)

def test_ext_is_gpr8_invalid_arg():
	with pytest.raises(ValueError):
		RegisterExt.is_gpr8(1234)

def test_ext_is_gpr16_invalid_arg():
	with pytest.raises(ValueError):
		RegisterExt.is_gpr16(1234)

def test_ext_is_gpr32_invalid_arg():
	with pytest.raises(ValueError):
		RegisterExt.is_gpr32(1234)

def test_ext_is_gpr64_invalid_arg():
	with pytest.raises(ValueError):
		RegisterExt.is_gpr64(1234)

def test_ext_is_xmm_invalid_arg():
	with pytest.raises(ValueError):
		RegisterExt.is_xmm(1234)

def test_ext_is_ymm_invalid_arg():
	with pytest.raises(ValueError):
		RegisterExt.is_ymm(1234)

def test_ext_is_zmm_invalid_arg():
	with pytest.raises(ValueError):
		RegisterExt.is_zmm(1234)

def test_ext_is_vector_register_invalid_arg():
	with pytest.raises(ValueError):
		RegisterExt.is_vector_register(1234)

def test_ext_is_ip_invalid_arg():
	with pytest.raises(ValueError):
		RegisterExt.is_ip(1234)

def test_ext_is_k_invalid_arg():
	with pytest.raises(ValueError):
		RegisterExt.is_k(1234)

def test_ext_is_cr_invalid_arg():
	with pytest.raises(ValueError):
		RegisterExt.is_cr(1234)

def test_ext_is_dr_invalid_arg():
	with pytest.raises(ValueError):
		RegisterExt.is_dr(1234)

def test_ext_is_tr_invalid_arg():
	with pytest.raises(ValueError):
		RegisterExt.is_tr(1234)

def test_ext_is_st_invalid_arg():
	with pytest.raises(ValueError):
		RegisterExt.is_st(1234)

def test_ext_is_bnd_invalid_arg():
	with pytest.raises(ValueError):
		RegisterExt.is_bnd(1234)

def test_ext_is_mm_invalid_arg():
	with pytest.raises(ValueError):
		RegisterExt.is_mm(1234)

def test_ext_is_tmm_invalid_arg():
	with pytest.raises(ValueError):
		RegisterExt.is_tmm(1234)



# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Mnemonic condition code selector (eg. ``JB`` / ``JC`` / ``JNAE``)
"""

B: int = 0
"""
``JB``, ``CMOVB``, ``SETB``
"""
C: int = 1
"""
``JC``, ``CMOVC``, ``SETC``
"""
NAE: int = 2
"""
``JNAE``, ``CMOVNAE``, ``SETNAE``
"""



# SPDX-License-Identifier: MIT
# Copyright (C) 2018-present iced project and contributors

# ⚠️This file was generated by GENERATOR!🦹‍♂️

# pylint: disable=invalid-name
# pylint: disable=line-too-long
# pylint: disable=too-many-lines

"""
Mnemonic condition code selector (eg. ``JNE`` / ``JNZ``)
"""

NE: int = 0
"""
``JNE``, ``CMOVNE``, ``SETNE``, ``LOOPNE``, ``REPNE``
"""
NZ: int = 1
"""
``JNZ``, ``CMOVNZ``, ``SETNZ``, ``LOOPNZ``, ``REPNZ``
"""
